package hogehoge;

import static org.lwjgl.opengl.GL11.*;

import java.util.Random;

import org.lwjgl.LWJGLException;
import org.lwjgl.Sys;
import org.lwjgl.opengl.Display;
import org.lwjgl.opengl.DisplayMode;

public class FPSTest {
	//表示する画像の数
	public final static int charNum = 100;

	//視体積の範囲
    private int        width = 1024;
    private int        height = 720;
    private int        depth = 10;

    //表示する画像のテクスチャー
    private Texture backTexture;
    private Texture[] charTexture = new Texture[charNum];

    //表示する画像の四方の座標
    float[][][] position = new float[charNum][2][4];

    //表示する画像の中心座標
    float[] x = new float[charNum];
    float[] y = new float[charNum];

    //FPS計算用
    private long time;
    private long timeDelta;
    private long fps;

    //乱数計算用
	Random rnd = new Random();

	//円周率
	public final static double PI = 3.14159265358979323846;

	//画像の四方座標を中心から計算する用
	public final static double rad = 45.0 / 180 * PI;

	//現在の回転角
	public double theta = 0;

	//thetaから算出したラジアン
	public double angle;

    public void start() {

        try {
        	//OpenGLのウインドウ生成
            Display.setDisplayMode(new DisplayMode(width, height));
            Display.setTitle("FPSTest");
            Display.create();
        } catch(LWJGLException e) {
            e.printStackTrace();
            return;
        }

        try {

        	//背景画像の読み込み
        	backTexture = new TextureLoader().loadTexture("background.png");

        	//キャラクター画像の読み込みと乱数による座標初期化
        	for(int i=0; i<charNum; i++){
        		charTexture[i] = new TextureLoader().loadTexture("char1.png");
        		x[i] = rnd.nextInt(1000);
        		y[i] = rnd.nextInt(700);
        	}

        	//プログラム開始時間の測定
        	time = getMillisecond();
        	fps = 0;

        	//以下OpenGLの初期化
        	glEnable(GL_TEXTURE_2D);

            glEnable(GL_CULL_FACE);

            glCullFace(GL_BACK);

			glEnable(GL_BLEND);

			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

            glMatrixMode(GL_PROJECTION);
            glLoadIdentity();

            //視体積の設定
            glOrtho(0, width, 0, height, 0, depth);

            glMatrixMode(GL_MODELVIEW);

            //メインループ
            while (!Display.isCloseRequested()) {
                glClear(GL_COLOR_BUFFER_BIT);

                //以下キャラクターの回転処理
                theta++;
                angle = theta / 180 * PI;

                for(int i=0; i<charNum; i++){
					position[i][0][0] = (float)(x[i] + (double)(30 * Math.cos(angle+(PI-rad))));
					position[i][0][1] = (float)(x[i] + (double)(30 * Math.cos(angle-(PI-rad))));
					position[i][0][2] = (float)(x[i] + (double)(30 * Math.cos(angle-rad)));
					position[i][0][3] = (float)(x[i] + (double)(30 * Math.cos(angle+rad)));

					position[i][1][0] = (float)(y[i] + (double)(30 * Math.sin(angle+(PI-rad))));
					position[i][1][1] = (float)(y[i] + (double)(30 * Math.sin(angle-(PI-rad))));
					position[i][1][2] = (float)(y[i] + (double)(30 * Math.sin(angle-rad)));
					position[i][1][3] = (float)(y[i] + (double)(30 * Math.sin(angle+rad)));
                }

                //画像の描画
                draw();

                //FPS計算
                calcFps();

                //画面へ反映
                Display.update();

            }
        } catch(Exception e) {
            e.printStackTrace();
        } finally {
            Display.destroy();
        }
    }

    private void draw() {
    	glLoadIdentity();

    	//色の設定
    	glColor4f(1f, 1f, 1f, 1f);

    	//背景の描画
    	drawPicture(backTexture, 0, height);

    	//キャラクターの描画
    	for(int i=0; i<charNum; i++){
    		drawRotatePicture(charTexture[i],
    				position[i][0][0], position[i][0][1], position[i][0][2], position[i][0][3],
    				position[i][1][0], position[i][1][1], position[i][1][2], position[i][1][3]);
    	}
    }

    private void drawPicture(Texture texture, float x, float y) {

    	//テクスチャーの適応
    	texture.bind();

    	//四角形の作成
    	//glVertex3fで座標を指定
    	glBegin(GL_QUADS);

    	float x2 = x + texture.getWidth();
    	float y2 = y - texture.getHeight();

    	//各座標にテクスチャーを合わせる
    	texture.point(texture.getWidth(), 0);
    	glVertex3f(x2, y, 0);

    	texture.point(0, 0);
    	glVertex3f(x, y, 0);

    	texture.point(0, texture.getHeight());
    	glVertex3f(x, y2, 0);

    	texture.point(texture.getWidth(), texture.getHeight());
    	glVertex3f(x2, y2, 0);

    	glEnd();

    }

    private void drawRotatePicture(Texture texture,
    		float x1, float x2, float x3, float x4,
    		float y1, float y2, float y3, float y4){

    	texture.bind();

    	glBegin(GL_QUADS);

    	texture.point(0, 0);
    	glVertex3f(x1, y1, 0);

    	texture.point(0, texture.getHeight());
    	glVertex3f(x2, y2, 0);

    	texture.point(texture.getWidth(), texture.getHeight());
    	glVertex3f(x3, y3, 0);

    	texture.point(texture.getWidth(), 0);
    	glVertex3f(x4, y4, 0);

    	glEnd();
    }

    private void calcFps() {
    	long now = getMillisecond();

    	timeDelta += now - time;

    	time = now;

    	if (1000 <= timeDelta) {
    		//１秒ごとにキャラクターの位置をランダムに変化
        	for(int i=0; i<charNum; i++){
        		x[i] = rnd.nextInt(1000);
        		y[i] = rnd.nextInt(700);
        	}

    		timeDelta -= 1000;

    		//タイトルにFPSを表示
    		Display.setTitle("FPSText  FPS:" + fps);

    		fps = 1;
    	} else {
    		fps++;
    	}
    }

    public static long getMillisecond() {
    	// 現在の時間をミリ秒で返す
    	return (Sys.getTime() * 1000) / Sys.getTimerResolution();
    }

    public static void main(String[] args) {
        new FPSTest().start();
    }
}
